﻿using Microsoft.Practices.ServiceLocation;
using Syncfusion.Windows.Tools.Controls;
using SyncfusionDockingDemo.Interfaces;
using System;
using System.Collections;
using System.Collections.Specialized;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

namespace SyncfusionDockingDemo
{
    /// <summary>
    /// Interaction logic for DockingManager.xaml
    /// </summary>
    public partial class DockingAdapter : UserControl
    {
        public DockingAdapter()
        {
            InitializeComponent();
            dockingManager.Loaded += dockingManager_Loaded;
            dockingManager.DockStateChanged += dockingManager_DockStateChanged;
        }

        private void dockingManager_DockStateChanged(FrameworkElement sender, DockStateEventArgs e)
        {
            switch (sender.Name)
            {
                case "favoritesDockWindow":
                    FavoritesDockState = e.NewState;
                    break;

                case "casesDockWindow":
                    CasesDockState = e.NewState;
                    break;

                case "queryHistoryDockWindow":
                    QueryHistoryDockState = e.NewState;
                    break;

                case "queryListDockWindow":
                    QueryListDockState = e.NewState;
                    break;
            }
        }

        private void dockingManager_Loaded(object sender, RoutedEventArgs e)
        {
            ((DocumentContainer)dockingManager.DocContainer).AddTabDocumentAtLast = true;
        }

        #region Active Document Dependency Property

        public IDockElement ActiveDocument
        {
            get { return (IDockElement)GetValue(ActiveDocumentProperty); }
            set { SetValue(ActiveDocumentProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ActiveDocument.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ActiveDocumentProperty =
            DependencyProperty.Register("ActiveDocument", typeof(IDockElement), typeof(DockingAdapter), new PropertyMetadata(null, new PropertyChangedCallback(onActiveDocumentChanged)));

        private static void onActiveDocumentChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
        {
            DockingAdapter adapter = sender as DockingAdapter;
            if (adapter != null)
            {
                foreach (FrameworkElement element in adapter.dockingManager.Children)
                {
                    if (element is ContentControl)
                    {
                        ContentControl control = element as ContentControl;
                        if (control != null && control.Content == args.NewValue)
                        {
                            adapter.dockingManager.ActiveWindow = control;
                            break;
                        }
                    }
                }
            }
        }

        #endregion

        #region ItemsSource Dependency Property

        public IList ItemsSource
        {
            get { return (IList)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ItemsSource.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ItemsSourceProperty =
            DependencyProperty.Register("ItemsSource", typeof(IList), typeof(DockingAdapter), new PropertyMetadata(null));

        #endregion

        #region FavoritesDockState Dependency Property

        public DockState FavoritesDockState
        {
            get { return (DockState)GetValue(FavoritesDockStateProperty); }
            set { SetValue(FavoritesDockStateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FavoritesDockState.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FavoritesDockStateProperty =
            DependencyProperty.Register("FavoritesDockState", typeof(DockState), typeof(DockingAdapter), new PropertyMetadata(DockState.Dock, onFavoritesDockStateChanged));

        private static void onFavoritesDockStateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DockState newValue = (DockState)e.NewValue;
            DockingAdapter da = d as DockingAdapter;
            if (DockingManager.GetState(da.favoritesDockWindow) != newValue)
            {
                da.FavoritesDockState = newValue;
                DockingManager.SetState(da.favoritesDockWindow, newValue);
            }
        }

        #endregion

        #region CaseDockState Dependency Property

        public DockState CasesDockState
        {
            get { return (DockState)GetValue(CasesDockStateProperty); }
            set { SetValue(CasesDockStateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CasesDockState.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CasesDockStateProperty =
            DependencyProperty.Register("CasesDockState", typeof(DockState), typeof(DockingAdapter), new PropertyMetadata(DockState.Dock, onCasesDockStateChanged));

        private static void onCasesDockStateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DockState newValue = (DockState)e.NewValue;
            DockingAdapter da = d as DockingAdapter;
            if (DockingManager.GetState(da.casesDockWindow) != newValue)
            {
                da.CasesDockState = newValue;
                DockingManager.SetState(da.casesDockWindow, newValue);
            }
        }

        #endregion

        #region QueryHistoryDockState Dependency Property

        public DockState QueryHistoryDockState
        {
            get { return (DockState)GetValue(QueryHistoryDockStateProperty); }
            set { SetValue(QueryHistoryDockStateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for HistoryDockState.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty QueryHistoryDockStateProperty =
            DependencyProperty.Register("QueryHistoryDockState", typeof(DockState), typeof(DockingAdapter), new PropertyMetadata(DockState.Dock, onQueryHistoryDockStateChanged));

        private static void onQueryHistoryDockStateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DockState newValue = (DockState)e.NewValue;
            DockingAdapter da = d as DockingAdapter;
            if (DockingManager.GetState(da.queryHistoryDockWindow) != newValue)
            {
                da.QueryHistoryDockState = newValue;
                DockingManager.SetState(da.queryHistoryDockWindow, newValue);
            }
        }

        #endregion

        #region QueryListDockState Dependency Property


        public DockState QueryListDockState
        {
            get { return (DockState)GetValue(QueryListDockStateProperty); }
            set { SetValue(QueryListDockStateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for QueryListDockState.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty QueryListDockStateProperty =
            DependencyProperty.Register("QueryListDockState", typeof(DockState), typeof(DockingAdapter), new PropertyMetadata(DockState.Dock, onQueryListDockStateChanged));

        private static void onQueryListDockStateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DockState newValue = (DockState)e.NewValue;
            DockingAdapter da = d as DockingAdapter;
            if (DockingManager.GetState(da.queryListDockWindow) != newValue)
            {
                da.QueryListDockState = newValue;
                DockingManager.SetState(da.queryListDockWindow, newValue);
            }
        }


        #endregion

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            if (e.Property.Name == "ItemsSource")
            {
                if (e.OldValue != null)
                {
                    var oldcollection = e.OldValue as INotifyCollectionChanged;
                    oldcollection.CollectionChanged -= CollectionChanged;
                }

                if (e.NewValue != null)
                {
                    ((DocumentContainer)dockingManager.DocContainer).AddTabDocumentAtLast = true;
                    var newcollection = e.NewValue as INotifyCollectionChanged;
                    foreach (var viewModel in ((IList)e.NewValue))
                    {
                        if (viewModel is IDockElement)
                        {
                            FrameworkElement control = viewLocator(viewModel);
                            DockingManager.SetHeader(control, ((IDockElement)viewModel).Header);
                            DockingManager.SetState(control, DockState.Document);
                            dockingManager.Children.Add(control);
                        }
                    }
                    newcollection.CollectionChanged += CollectionChanged;
                }
            }
            base.OnPropertyChanged(e);
        }

        private void CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.OldItems != null)
            {
                foreach (var item in e.OldItems)
                {
                    ContentControl control = (from ContentControl element in dockingManager.Children
                                              where element.Content == item
                                              select element).FirstOrDefault();
                    dockingManager.Children.Remove(control);
                }
            }

            if (e.NewItems != null)
            {
                foreach (IDockElement viewModel in e.NewItems)
                {
                    if (viewModel is IDockElement)
                    {
                        try
                        {
                            //UserControl view = viewLocator(viewModel);
                            ContentControl view = new ContentControl() { Content = viewModel };
                            DockingManager.SetHeader(view, viewModel.Header);
                            DockingManager.SetState(view, DockState.Document);
                            dockingManager.Children.Add(view);
                        }
                        catch (Exception ex)
                        {
                            System.Diagnostics.Trace.Write(ex);
                        }
                    }
                }
            }
        }

        private void dockingManager_ActiveWindowChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (e.NewValue is ContentControl)
            {
                var content = e.NewValue as ContentControl;
                IDockElement dataContext = content.DataContext as IDockElement;
                if (dataContext != null)
                    ActiveDocument = (IDockElement)content.Content;
            }
        }

        private UserControl viewLocator(object viewModel)
        {
            var viewTypeName = viewModel.GetType().FullName.Replace("ViewModels", "Views").Replace("ViewModel", "");
            var viewType = viewModel.GetType().Assembly.GetType(viewTypeName);

            var view = (UserControl)ServiceLocator.Current.TryResolve(viewType);
            view.DataContext = viewModel;

            return view;
        }
    }
}
