﻿using Microsoft.Practices.Unity;
using Prism.Commands;
using Prism.Mvvm;
using Syncfusion.Windows.Tools.Controls;
using SyncfusionDockingDemo.Interfaces;
using System.Collections.ObjectModel;

namespace SyncfusionDockingDemo.ViewModels
{
    public class MainWindowViewModel : BindableBase
    {
        [Dependency]
        public IUnityContainer Container { get; set; }

        #region Commands

        public DelegateCommand ToggleFavoritesCommand { get; private set; }

        public DelegateCommand ToggleQueryHistoryCommand { get; private set; }

        public DelegateCommand ToggleCasesCommand { get; private set; }

        public DelegateCommand ToggleQueryListCommand { get; private set; }

        public DelegateCommand AddContentCommand { get; private set; }

        #endregion

        private string _title = "Prism Unity Docking Application";
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }

        #region Favorites Region

        private DockState favoritesOldDockState;
        public DockState FavoritesOldDockState
        {
            get { return favoritesOldDockState; }
            set { SetProperty(ref favoritesOldDockState, value); }
        }

        private DockState favoritesDockState;
        public DockState FavoritesDockState
        {
            get { return favoritesDockState; }
            set
            {
                if (FavoritesOldDockState != value && FavoritesOldDockState != DockState.Hidden)
                    FavoritesOldDockState = FavoritesDockState;
                SetProperty(ref favoritesDockState, value);
                OnPropertyChanged("FavoritesMenuItemText");
            }
        }

        public string FavoritesMenuItemText
        {
            get
            {
                if (FavoritesDockState != DockState.Hidden)
                    return "Hide Favorites";
                else
                    return "Show Favorites";
            }
        }

        #endregion

        #region Query List Region

        private DockState queryListOldDockState;
        public DockState QueryListOldDockState
        {
            get { return queryListOldDockState; }
            set { SetProperty(ref queryListOldDockState, value); }
        }

        private DockState queryListDockState;
        public DockState QueryListDockState
        {
            get { return queryListDockState; }
            set
            {
                if (QueryListOldDockState != value && QueryListOldDockState != DockState.Hidden)
                    QueryListOldDockState = QueryListDockState;
                SetProperty(ref queryListDockState, value);
                OnPropertyChanged("QueryListMenuItemText");
            }
        }

        public string QueryListMenuItemText
        {
            get
            {
                if (QueryListDockState != DockState.Hidden)
                    return "Hide Query List";
                else
                    return "Show Query List";
            }
        }

        #endregion

        #region Query History Region

        private DockState queryHistoryOldDockState;
        public DockState QueryHistoryOldDockState
        {
            get { return queryHistoryOldDockState; }
            set { SetProperty(ref queryHistoryOldDockState, value); }
        }

        private DockState queryHistoryDockState;
        public DockState QueryHistoryDockState
        {
            get { return queryHistoryDockState; }
            set
            {
                if (QueryHistoryOldDockState != value && QueryHistoryOldDockState != DockState.Hidden)
                    QueryHistoryOldDockState = QueryHistoryDockState;
                SetProperty(ref queryHistoryDockState, value);
                OnPropertyChanged("QueryHistoryMenuItemText");
            }
        }

        public string QueryHistoryMenuItemText
        {
            get
            {
                if (QueryHistoryDockState != DockState.Hidden)
                    return "Hide Query History";
                else
                    return "Show Query History";
            }
        }

        #endregion

        #region Cases Region

        private DockState casesOldDockState;
        public DockState CasesOldDockState
        {
            get { return casesOldDockState; }
            set { SetProperty(ref casesOldDockState, value); }
        }

        private DockState casesDockState;
        public DockState CasesDockState
        {
            get { return casesDockState; }
            set
            {
                if (CasesOldDockState != value && CasesOldDockState != DockState.Hidden)
                    CasesOldDockState = CasesDockState;
                SetProperty(ref casesDockState, value);
                OnPropertyChanged("CasesMenuItemText");
            }
        }

        public string CasesMenuItemText
        {
            get
            {
                if (CasesDockState != DockState.Hidden)
                    return "Hide Cases";
                else
                    return "Show Cases";
            }
        }

        #endregion

        public ObservableCollection<IDockElement> DockWindows { get; set; }

        public MainWindowViewModel()
        {
            DockWindows = new ObservableCollection<IDockElement>();

            ToggleFavoritesCommand = new DelegateCommand(() =>
            {
                if (FavoritesDockState != DockState.Hidden)
                    FavoritesDockState = DockState.Hidden;
                else
                    FavoritesDockState = FavoritesOldDockState;
            });
            ToggleQueryListCommand = new DelegateCommand(() =>
            {
                if (QueryListDockState != DockState.Hidden)
                    QueryListDockState = DockState.Hidden;
                else
                    QueryListDockState = QueryListOldDockState;
            });
            ToggleQueryHistoryCommand = new DelegateCommand(() =>
            {
                if (QueryHistoryDockState != DockState.Hidden)
                    QueryHistoryDockState = DockState.Hidden;
                else
                    QueryHistoryDockState = QueryHistoryOldDockState;
            });
            ToggleCasesCommand = new DelegateCommand(() =>
            {
                if (CasesDockState != DockState.Hidden)
                    CasesDockState = DockState.Hidden;
                else
                    CasesDockState = CasesOldDockState;
            });
            AddContentCommand = new DelegateCommand(() =>
            {
                DockWindows.Add(Container.Resolve<ContentPageViewModel>());
            });
        }
    }
}
